@@ -46,6 +46,7 @@ typedef enum {
     MMC_S_MODEM_3GPP_USSD,
     MMC_S_MODEM_3GPP_PROFILE_MANAGER,
     MMC_S_MODEM_CDMA,
+    MMC_S_MODEM_CELL_BROADCAST,
     MMC_S_MODEM_SIM,
     MMC_S_MODEM_BEARER,
     MMC_S_MODEM_TIME,
@@ -83,6 +84,9 @@ typedef enum {
     MMC_S_SMS_PROPERTIES,
     MMC_S_SIM_GENERAL,
     MMC_S_SIM_PROPERTIES,
+    MMC_S_CBM_GENERAL,
+    MMC_S_CBM_CONTENT,
+    MMC_S_CBM_PROPERTIES,
 } MmcS;
 
 /******************************************************************************/
@@ -138,6 +142,10 @@ typedef enum {
     MMC_F_3GPP_OPERATOR_NAME,
     MMC_F_3GPP_REGISTRATION,
     MMC_F_3GPP_PACKET_SERVICE_STATE,
+    MMC_F_3GPP_NETWORK_REJECTION_ERROR,
+    MMC_F_3GPP_NETWORK_REJECTION_OPERATOR_ID,
+    MMC_F_3GPP_NETWORK_REJECTION_OPERATOR_NAME,
+    MMC_F_3GPP_NETWORK_REJECTION_ACCESS_TECHNOLOGY,
     MMC_F_3GPP_PCO,
     /* 3GPP EPS section */
     MMC_F_3GPP_EPS_UE_MODE,
@@ -166,6 +174,8 @@ typedef enum {
     MMC_F_CDMA_REGISTRATION_CDMA1X,
     MMC_F_CDMA_REGISTRATION_EVDO,
     MMC_F_CDMA_ACTIVATION,
+    /* CellBroadcast section */
+    MMC_F_CELL_BROADCAST_CHANNELS,
     /* SIM section */
     MMC_F_SIM_PATH,
     MMC_F_SIM_PRIMARY_SLOT,
@@ -204,6 +214,7 @@ typedef enum {
     MMC_F_SIGNAL_LTE_RSRP,
     MMC_F_SIGNAL_LTE_SNR,
     MMC_F_SIGNAL_LTE_ERROR_RATE,
+    MMC_F_SIGNAL_5G_RSSI,
     MMC_F_SIGNAL_5G_RSRQ,
     MMC_F_SIGNAL_5G_RSRP,
     MMC_F_SIGNAL_5G_SNR,
@@ -299,6 +310,11 @@ typedef enum {
     MMC_F_CALL_AUDIO_FORMAT_ENCODING,
     MMC_F_CALL_AUDIO_FORMAT_RESOLUTION,
     MMC_F_CALL_AUDIO_FORMAT_RATE,
+    MMC_F_CBM_GENERAL_DBUS_PATH,
+    MMC_F_CBM_CONTENT_TEXT,
+    MMC_F_CBM_PROPERTIES_CHANNEL,
+    MMC_F_CBM_PROPERTIES_UPDATE,
+    MMC_F_CBM_PROPERTIES_MESSAGE_CODE,
     MMC_F_SMS_GENERAL_DBUS_PATH,
     MMC_F_SMS_CONTENT_NUMBER,
     MMC_F_SMS_CONTENT_TEXT,
@@ -336,6 +352,7 @@ typedef enum {
     MMC_F_MODEM_LIST_DBUS_PATH,
     MMC_F_SMS_LIST_DBUS_PATH,
     MMC_F_CALL_LIST_DBUS_PATH,
+    MMC_F_CBM_LIST_DBUS_PATH,
 } MmcF;
 
 /******************************************************************************/
