@@ -44,7 +44,7 @@ static MMBaseManager *manager;
 static gboolean
 quit_cb (gpointer user_data)
 {
-    mm_info ("caught signal, shutting down...");
+    mm_msg ("caught signal, shutting down...");
 
     if (manager)
         g_object_set (manager, MM_BASE_MANAGER_CONNECTION, NULL, NULL);
@@ -61,8 +61,13 @@ quit_cb (gpointer user_data)
 static void
 sleeping_cb (MMSleepMonitor *sleep_monitor)
 {
-    mm_dbg ("removing devices... (sleeping)");
-    mm_base_manager_shutdown (manager, FALSE);
+    if (mm_context_get_test_low_power_suspend_resume ()) {
+        mm_dbg ("removing devices and setting them in low power mode... (sleeping)");
+        mm_base_manager_shutdown (manager, TRUE, TRUE, TRUE);
+    } else {
+        mm_dbg ("removing devices... (sleeping)");
+        mm_base_manager_shutdown (manager, FALSE, FALSE, TRUE);
+    }
 }
 
 static void
@@ -72,6 +77,15 @@ resuming_cb (MMSleepMonitor *sleep_monitor)
     mm_base_manager_start (manager, FALSE);
 }
 
+static void
+sleeping_quick_cb (MMSleepMonitor *sleep_monitor)
+{
+    if (mm_context_get_test_low_power_suspend_resume ()) {
+        mm_dbg ("setting modem in low power mode... (sleeping)");
+        mm_base_manager_shutdown (manager, TRUE, TRUE, FALSE);
+    }
+}
+
 static void
 resuming_quick_cb (MMSleepMonitor *sleep_monitor)
 {
@@ -93,7 +107,9 @@ bus_acquired_cb (GDBusConnection *connection,
     /* Create Manager object */
     g_assert (!manager);
     manager = mm_base_manager_new (connection,
+#if !defined WITH_BUILTIN_PLUGINS
                                    mm_context_get_test_plugin_dir (),
+#endif
                                    !mm_context_get_no_auto_scan (),
                                    mm_context_get_filter_policy (),
                                    mm_context_get_initial_kernel_events (),
@@ -180,8 +196,8 @@ main (int argc, char *argv[])
     /* Early register all known errors */
     register_dbus_errors ();
 
-    mm_info ("ModemManager (version " MM_DIST_VERSION ") starting in %s bus...",
-             mm_context_get_test_session () ? "session" : "system");
+    mm_msg ("ModemManager (version " MM_DIST_VERSION ") starting in %s bus...",
+            mm_context_get_test_session () ? "session" : "system");
 
     /* Detect runtime charset conversion support */
     mm_modem_charsets_init ();
@@ -199,11 +215,12 @@ main (int argc, char *argv[])
     {
         MMSleepMonitor *sleep_monitor;
 
-        if (mm_context_get_test_no_suspend_resume())
+        if (mm_context_get_test_no_suspend_resume ())
             mm_dbg ("Suspend/resume support disabled at runtime");
-        else if (mm_context_get_test_quick_suspend_resume()) {
+        else if (mm_context_get_test_quick_suspend_resume ()) {
             mm_dbg ("Quick suspend/resume hooks enabled");
             sleep_monitor = mm_sleep_monitor_get ();
+            g_signal_connect (sleep_monitor, MM_SLEEP_MONITOR_SLEEPING, G_CALLBACK (sleeping_quick_cb), NULL);
             g_signal_connect (sleep_monitor, MM_SLEEP_MONITOR_RESUMING, G_CALLBACK (resuming_quick_cb), NULL);
         } else {
             mm_dbg ("Full suspend/resume hooks enabled");
@@ -226,7 +243,7 @@ main (int argc, char *argv[])
     if (manager) {
         GTimer *timer;
 
-        mm_base_manager_shutdown (manager, TRUE);
+        mm_base_manager_shutdown (manager, TRUE, FALSE, TRUE);
 
         /* Wait for all modems to be disabled and removed, but don't wait
          * forever: if disabling the modems takes longer than 20s, just
@@ -252,7 +269,7 @@ main (int argc, char *argv[])
 
     g_bus_unown_name (name_id);
 
-    mm_info ("ModemManager is shut down");
+    mm_msg ("ModemManager is shut down");
 
     mm_log_shutdown ();
 
