@@ -40,6 +40,8 @@
 #include <ModemManager-tags.h>
 
 #include <mm-errors-types.h>
+#include "mm-error-helpers.h"
+
 #include <mm-gdbus-manager.h>
 #if defined WITH_TESTS
 # include <mm-gdbus-test.h>
@@ -55,6 +57,9 @@
 #include "mm-filter.h"
 #include "mm-log-object.h"
 #include "mm-base-modem.h"
+#include "mm-iface-modem.h"
+
+#include "mm-dispatcher-modem-setup.h"
 
 static void initable_iface_init   (GInitableIface       *iface);
 static void log_object_iface_init (MMLogObjectInterface *iface);
@@ -68,7 +73,9 @@ enum {
     PROP_CONNECTION,
     PROP_AUTO_SCAN,
     PROP_FILTER_POLICY,
+#if !defined WITH_BUILTIN_PLUGINS
     PROP_PLUGIN_DIR,
+#endif
     PROP_INITIAL_KERNEL_EVENTS,
 #if defined WITH_TESTS
     PROP_ENABLE_TEST,
@@ -83,8 +90,10 @@ struct _MMBaseManagerPrivate {
     gboolean auto_scan;
     /* Filter policy (mask of enabled rules) */
     MMFilterRule filter_policy;
+#if !defined WITH_BUILTIN_PLUGINS
     /* Path to look for plugins */
     gchar *plugin_dir;
+#endif
     /* Path to the list of initial kernel events */
     gchar *initial_kernel_events;
     /* The authorization provider */
@@ -183,7 +192,7 @@ find_device_by_physdev_uid (MMBaseManager *self,
 
 typedef struct {
     MMBaseManager *self;
-    MMDevice *device;
+    MMDevice      *device;
 } FindDeviceSupportContext;
 
 static void
@@ -194,47 +203,121 @@ find_device_support_context_free (FindDeviceSupportContext *ctx)
     g_slice_free (FindDeviceSupportContext, ctx);
 }
 
+static void
+dispatcher_modem_setup_ready (MMDispatcherModemSetup    *dispatcher,
+                              GAsyncResult              *res,
+                              FindDeviceSupportContext  *ctx)
+{
+    g_autoptr(GError) error = NULL;
+
+    if (!mm_dispatcher_modem_setup_run_finish (dispatcher, res, &error)) {
+        if (!g_error_matches(error, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND))
+            mm_obj_warn (ctx->self, "couldn't run setup for device '%s': %s",
+                         mm_device_get_uid (ctx->device), error->message);
+        else
+            mm_obj_dbg (ctx->self, "no need to run setup for device '%s'",
+                        mm_device_get_uid (ctx->device));
+    } else
+        mm_obj_dbg (ctx->self, "setup for device '%s' finished",
+                    mm_device_get_uid (ctx->device));
+
+    /* launch async modem initialization */
+    mm_device_initialize_modem (ctx->device);
+
+    find_device_support_context_free (ctx);
+}
+
+static void
+modem_setup (FindDeviceSupportContext *ctx)
+{
+    guint                     n_port_infos = 0;
+    g_auto(GStrv)             modem_ports = NULL;
+    MMDispatcherModemSetup   *dispatcher;
+    MMModemPortInfo          *port_infos;
+    MMBaseModem              *modem = NULL;
+    GPtrArray                *aux;
+    guint                     i;
+
+    dispatcher = mm_dispatcher_modem_setup_get ();
+
+    modem = mm_device_peek_modem (ctx->device);
+
+    aux = g_ptr_array_new ();
+    port_infos = mm_base_modem_get_port_infos (modem, &n_port_infos);
+    for (i = 0; i < n_port_infos; i++) {
+        switch (port_infos[i].type) {
+            case MM_MODEM_PORT_TYPE_AT:
+            case MM_MODEM_PORT_TYPE_QMI:
+            case MM_MODEM_PORT_TYPE_MBIM:
+            case MM_MODEM_PORT_TYPE_XMMRPC:
+                g_ptr_array_add (aux, g_strdup (port_infos[i].name));
+                break;
+            case MM_MODEM_PORT_TYPE_UNKNOWN:
+            case MM_MODEM_PORT_TYPE_NET:
+            case MM_MODEM_PORT_TYPE_QCDM:
+            case MM_MODEM_PORT_TYPE_GPS:
+            case MM_MODEM_PORT_TYPE_AUDIO:
+            case MM_MODEM_PORT_TYPE_IGNORED:
+            default:
+                break;
+        }
+    }
+
+    mm_modem_port_info_array_free (port_infos, n_port_infos);
+    g_ptr_array_add (aux, NULL);
+    modem_ports = (GStrv) g_ptr_array_free (aux, FALSE);
+
+    mm_obj_msg (ctx->self, "running setup for device '%s'...",
+                mm_device_get_uid (ctx->device));
+    mm_dispatcher_modem_setup_run (dispatcher,
+                                   mm_base_modem_get_vendor_id (modem),
+                                   mm_base_modem_get_product_id (modem),
+                                   mm_base_modem_get_device (modem),
+                                   modem_ports,
+                                   NULL,
+                                   (GAsyncReadyCallback)dispatcher_modem_setup_ready,
+                                   ctx);
+}
+
 static void
 device_support_check_ready (MMPluginManager          *plugin_manager,
                             GAsyncResult             *res,
                             FindDeviceSupportContext *ctx)
 {
-    GError   *error = NULL;
-    MMPlugin *plugin;
+    g_autoptr(GError)   error = NULL;
+    g_autoptr(MMPlugin) plugin = NULL;
 
     /* If the device support check fails, either with an error, or afterwards
-     * when trying to create a modem object, we must remove the MMDevice from
-     * the tracking table of devices, so that a manual scan request afterwards
-     * re-scans all ports. */
+     * when trying to create a modem object, we must reset the port probe list
+     * in the MMDevice, so that a manual scan request afterwards re-scans all
+     * ports. */
 
     /* Receive plugin result from the plugin manager */
     plugin = mm_plugin_manager_device_support_check_finish (plugin_manager, res, &error);
     if (!plugin) {
-        mm_obj_info (ctx->self, "couldn't check support for device '%s': %s",
-                     mm_device_get_uid (ctx->device), error->message);
-        g_error_free (error);
-        g_hash_table_remove (ctx->self->priv->devices, mm_device_get_uid (ctx->device));
+        mm_obj_msg (ctx->self, "couldn't check support for device '%s': %s",
+                    mm_device_get_uid (ctx->device), error->message);
+        mm_device_reset_port_probe_list (ctx->device);
         find_device_support_context_free (ctx);
         return;
     }
 
     /* Set the plugin as the one expected in the device */
     mm_device_set_plugin (ctx->device, G_OBJECT (plugin));
-    g_object_unref (plugin);
 
     if (!mm_device_create_modem (ctx->device, &error)) {
         mm_obj_warn (ctx->self, "couldn't create modem for device '%s': %s",
                      mm_device_get_uid (ctx->device), error->message);
-        g_error_free (error);
-        g_hash_table_remove (ctx->self->priv->devices, mm_device_get_uid (ctx->device));
+        mm_device_reset_port_probe_list (ctx->device);
         find_device_support_context_free (ctx);
         return;
     }
 
     /* Modem now created */
-    mm_obj_info (ctx->self, "modem for device '%s' successfully created",
-                 mm_device_get_uid (ctx->device));
-    find_device_support_context_free (ctx);
+    mm_obj_msg (ctx->self, "modem for device '%s' successfully created",
+                mm_device_get_uid (ctx->device));
+
+    modem_setup (ctx);
 }
 
 static gboolean is_device_inhibited           (MMBaseManager  *self,
@@ -273,10 +356,10 @@ device_removed (MMBaseManager *self,
      * ourselves. */
     g_object_ref (device);
 
-    mm_obj_info (self, "port %s released by device '%s'", name, mm_device_get_uid (device));
+    mm_obj_msg (self, "port %s released by device '%s'", name, mm_device_get_uid (device));
     mm_device_release_port_name (device, subsystem, name);
 
-    /* If port probe list gets empty, remove the device object iself */
+    /* If port probe list gets empty, remove the device object itself */
     if (!mm_device_peek_port_probe_list (device)) {
         mm_obj_dbg (self, "removing empty device '%s'", mm_device_get_uid (device));
         if (mm_plugin_manager_device_support_check_cancel (self->priv->plugin_manager, device))
@@ -289,6 +372,170 @@ device_removed (MMBaseManager *self,
     }
 }
 
+static void
+device_support_check_add_all_ports (MMBaseManager  *self,
+                                    MMDevice       *device)
+{
+    FindDeviceSupportContext *ctx;
+    GList                    *l;
+
+    g_assert (!mm_plugin_manager_device_support_check_ongoing (self->priv->plugin_manager, device));
+
+    /* There is no running device support context, so we launch a new one */
+    ctx = g_slice_new (FindDeviceSupportContext);
+    ctx->self = g_object_ref (self);
+    ctx->device = g_object_ref (device);
+    mm_plugin_manager_device_support_check (self->priv->plugin_manager,
+                                            device,
+                                            (GAsyncReadyCallback) device_support_check_ready,
+                                            ctx);
+
+    g_assert (mm_plugin_manager_device_support_check_ongoing (self->priv->plugin_manager, device));
+
+    /* Iterate all known ports and notify them one by one */
+    for (l = mm_device_peek_port_probe_list (device); l; l = g_list_next (l)) {
+        mm_plugin_manager_device_support_check_add_port (self->priv->plugin_manager,
+                                                         device,
+                                                         mm_port_probe_peek_port (MM_PORT_PROBE (l->data)));
+    }
+}
+
+static void
+device_support_check_add_single_port (MMBaseManager  *self,
+                                      MMDevice       *device,
+                                      MMKernelDevice *port)
+{
+    FindDeviceSupportContext *ctx;
+    gboolean                  added;
+
+    /* Try to add the port to an already running device support context */
+    if (mm_plugin_manager_device_support_check_add_port (self->priv->plugin_manager, device, port))
+        return;
+
+    /* There is no running device support context, so we launch a new one */
+    ctx = g_slice_new (FindDeviceSupportContext);
+    ctx->self = g_object_ref (self);
+    ctx->device = g_object_ref (device);
+    mm_plugin_manager_device_support_check (self->priv->plugin_manager,
+                                            device,
+                                            (GAsyncReadyCallback) device_support_check_ready,
+                                            ctx);
+
+    /* Retry now, which should never fail */
+    added = mm_plugin_manager_device_support_check_add_port (self->priv->plugin_manager, device, port);
+    g_assert (added);
+}
+
+static void
+existing_port (MMBaseManager  *self,
+               MMDevice       *device,
+               MMKernelDevice *port)
+{
+    const gchar *name;
+    const gchar *uid;
+
+    name = mm_kernel_device_get_name (port);
+    uid = mm_device_get_uid (device);
+
+    mm_obj_dbg (self, "port %s already available in device '%s'", name, uid);
+
+    /* If the port exists already in the device, and we already have a valid modem
+     * object created, there is no point in processing this event, we can warn about
+     * the situation and bail out. This may happen e.g. if a manual scan is requested
+     * after having successfully finished the last probing operation. */
+    if (mm_device_peek_plugin (device) && mm_device_peek_modem (device)) {
+        mm_obj_dbg (self, "ignoring port %s as device '%s' already has a valid modem", name, uid);
+        return;
+    }
+
+    /* The port exists in the device, but we don't have a valid modem associated with
+     * it. This may happen if a manual scan is requested after having failed the last
+     * device probing operation. We need to relaunch a new device probing task, unless
+     * one is already running. */
+    device_support_check_add_single_port (self, device, port);
+}
+
+static void
+additional_port (MMBaseManager  *self,
+                 MMDevice       *device,
+                 MMKernelDevice *port)
+{
+    MMBaseModem *modem;
+    const gchar *name;
+    const gchar *uid;
+
+    name = mm_kernel_device_get_name (port);
+    uid = mm_device_get_uid (device);
+
+    mm_obj_dbg (self, "additional port %s in device '%s'", name, uid);
+
+    /* Do nothing if the device is ignoring the new port */
+    if (!mm_device_grab_port (device, port))
+        return;
+
+    /* If there is an ongoing support check, we can add the single port right away */
+    if (mm_plugin_manager_device_support_check_ongoing (self->priv->plugin_manager, device)) {
+        device_support_check_add_single_port (self, device, port);
+        return;
+    }
+
+    mm_obj_warn (self, "additional port %s in device '%s' added after device probing has already finished", name, uid);
+
+    /* If there is no plugin assigned in the device, it means the device probing had failed earlier,
+     * for example if none of the ports that were used for the device probing were usable control ports */
+    if (!mm_device_peek_plugin (device)) {
+        g_assert (!mm_device_peek_modem (device));
+        mm_obj_info (self, "last device '%s' probing had failed, will retry", uid);
+        device_support_check_add_all_ports (self, device);
+        return;
+    }
+
+    /* If there is no modem object assigned to the device, it means the modem object creation had failed earlier,
+     * for example if a required port was missing (e.g. missing net port in a QMI or MBIM modem) */
+    modem = mm_device_peek_modem (device);
+    if (!modem) {
+        mm_obj_info (self, "last modem object creation in device '%s' had failed, will retry", uid);
+        device_support_check_add_all_ports (self, device);
+        return;
+    }
+
+    mm_obj_info (self, "last modem object creation in device '%s' succeeded, but we have a new port addition, will retry", uid);
+    g_cancellable_cancel (mm_base_modem_peek_cancellable (modem));
+    mm_device_remove_modem (device);
+    device_support_check_add_all_ports (self, device);
+}
+
+static void
+first_port (MMBaseManager  *self,
+            MMKernelDevice *port,
+            gboolean        hotplugged)
+{
+    g_autoptr(MMDevice)  device = NULL;
+    const gchar         *name;
+    const gchar         *uid;
+    const gchar         *physdev;
+
+    name = mm_kernel_device_get_name (port);
+    uid = mm_kernel_device_get_physdev_uid (port);
+
+    mm_obj_dbg (self, "port %s is first in device '%s'", name, uid);
+
+    physdev = mm_kernel_device_get_physdev_sysfs_path (port);
+    device = mm_device_new (uid, physdev, hotplugged, FALSE, self->priv->object_manager);
+
+    /* If the device is ignoring the new port, discard the new device as well */
+    if (!mm_device_grab_port (device, port)) {
+        mm_obj_dbg (self, "discarding device '%s' as the first port is not grabbed", uid);
+        return;
+    }
+
+    /* Store the device */
+    g_hash_table_insert (self->priv->devices, g_strdup (uid), g_object_ref (device));
+
+    /* And start device support check */
+    device_support_check_add_single_port (self, device, port);
+}
+
 static void
 device_added (MMBaseManager  *self,
               MMKernelDevice *port,
@@ -296,8 +543,8 @@ device_added (MMBaseManager  *self,
               gboolean        manual_scan)
 {
     MMDevice    *device;
-    const gchar *physdev_uid;
     const gchar *name;
+    const gchar *uid;
 
     g_return_if_fail (port != NULL);
 
@@ -326,15 +573,15 @@ device_added (MMBaseManager  *self,
 
     /* Get the port's physical device's uid. All ports of the same physical
      * device will share the same uid. */
-    physdev_uid = mm_kernel_device_get_physdev_uid (port);
-    g_assert (physdev_uid);
+    uid = mm_kernel_device_get_physdev_uid (port);
+    g_assert (uid);
 
     /* If the device is inhibited, do nothing else */
-    if (is_device_inhibited (self, physdev_uid)) {
+    if (is_device_inhibited (self, uid)) {
         /* Note: we will not report as hotplugged an inhibited device port
          * because we don't know what was done with the port out of our
          * context. */
-        device_inhibited_track_port (self, physdev_uid, port, manual_scan);
+        device_inhibited_track_port (self, uid, port, manual_scan);
         return;
     }
 
@@ -342,39 +589,22 @@ device_added (MMBaseManager  *self,
     if (!mm_filter_port (self->priv->filter, port, manual_scan))
         return;
 
-    /* If already added, ignore new event */
-    if (find_device_by_port (self, port)) {
-        mm_obj_dbg (self, "port %s already added", name);
+    /* If the port is already added, don't add it again */
+    device = find_device_by_port (self, port);
+    if (device) {
+        existing_port (self, device, port);
         return;
     }
 
     /* See if we already created an object to handle ports in this device */
-    device = find_device_by_physdev_uid (self, physdev_uid);
-    if (!device) {
-        FindDeviceSupportContext *ctx;
+    device = find_device_by_physdev_uid (self, uid);
+    if (device) {
+        additional_port (self, device, port);
+        return;
+    }
 
-        mm_obj_dbg (self, "port %s is first in device %s", name, physdev_uid);
-
-        /* Keep the device listed in the Manager */
-        device = mm_device_new (physdev_uid, hotplugged, FALSE, self->priv->object_manager);
-        g_hash_table_insert (self->priv->devices,
-                             g_strdup (physdev_uid),
-                             device);
-
-        /* Launch device support check */
-        ctx = g_slice_new (FindDeviceSupportContext);
-        ctx->self = g_object_ref (self);
-        ctx->device = g_object_ref (device);
-        mm_plugin_manager_device_support_check (
-            self->priv->plugin_manager,
-            device,
-            (GAsyncReadyCallback) device_support_check_ready,
-            ctx);
-    } else
-        mm_obj_dbg (self, "additional port %s in device %s", name, physdev_uid);
-
-    /* Grab the port in the existing device. */
-    mm_device_grab_port (device, port);
+    /* This is the first port in a new device */
+    first_port (self, port, hotplugged);
 }
 
 #if defined WITH_QRTR
@@ -643,39 +873,104 @@ mm_base_manager_start (MMBaseManager *self,
 
 /*****************************************************************************/
 
+typedef struct {
+    MMBaseManager *self;
+    gboolean       low_power;
+    gboolean       remove;
+} DisableContext;
+
 static void
-remove_disable_ready (MMBaseModem *modem,
-                      GAsyncResult *res,
-                      MMBaseManager *self)
+disable_context_free (DisableContext *ctx)
+{
+    g_object_unref (ctx->self);
+    g_slice_free (DisableContext, ctx);
+}
+
+static void
+remove_device_after_disable (MMBaseModem    *modem,
+                             DisableContext *ctx)
 {
     MMDevice *device;
 
-    /* We don't care about errors disabling at this point */
-    mm_base_modem_disable_finish (modem, res, NULL);
-
-    device = find_device_by_modem (self, modem);
+    device = find_device_by_modem (ctx->self, modem);
     if (device) {
         g_cancellable_cancel (mm_base_modem_peek_cancellable (modem));
         mm_device_remove_modem (device);
-        g_hash_table_remove (self->priv->devices, mm_device_get_uid (device));
+        g_hash_table_remove (ctx->self->priv->devices, mm_device_get_uid (device));
     }
+
+    disable_context_free (ctx);
 }
 
 static void
-foreach_disable (gpointer key,
-                 MMDevice *device,
-                 MMBaseManager *self)
+shutdown_low_power_ready (MMIfaceModem   *modem,
+                          GAsyncResult   *res,
+                          DisableContext *ctx)
 {
-    MMBaseModem *modem;
+    g_autoptr(GError) error = NULL;
+
+    if (!mm_iface_modem_set_power_state_finish (modem, res, NULL, &error))
+        mm_obj_info (ctx->self, "changing to low power state failed: %s", error->message);
+
+    if (ctx->remove)
+        remove_device_after_disable (MM_BASE_MODEM (modem), ctx);
+    else
+        disable_context_free (ctx);
+}
+
+static void
+shutdown_disable_ready (MMBaseModem    *modem,
+                        GAsyncResult   *res,
+                        DisableContext *ctx)
+{
+    g_autoptr(GError) error = NULL;
+
+    /* We don't care about errors disabling at this point */
+    if (!mm_base_modem_disable_finish (modem, res, &error)) {
+        mm_obj_info (ctx->self, "disabling modem failed: %s", error->message);
+    }
+    /* Bring the modem to low power mode if requested */
+    else if (ctx->low_power) {
+        mm_iface_modem_set_power_state (MM_IFACE_MODEM (modem),
+                                        MM_MODEM_POWER_STATE_LOW,
+                                        (GAsyncReadyCallback)shutdown_low_power_ready,
+                                        ctx);
+        return;
+    }
+
+    if (ctx->remove)
+        remove_device_after_disable (modem, ctx);
+    else
+        disable_context_free (ctx);
+}
+
+static void
+foreach_disable (gpointer        key,
+                 MMDevice       *device,
+                 DisableContext *foreach_ctx)
+{
+    MMBaseModem    *modem;
+    DisableContext *ctx;
 
     modem = mm_device_peek_modem (device);
-    if (modem)
-        mm_base_modem_disable (modem, (GAsyncReadyCallback)remove_disable_ready, self);
+    if (!modem)
+        return;
+
+    ctx = g_slice_new0 (DisableContext);
+    ctx->self = g_object_ref (foreach_ctx->self);
+    ctx->low_power = foreach_ctx->low_power;
+    ctx->remove = foreach_ctx->remove;
+
+    mm_base_modem_disable (modem,
+                           MM_BASE_MODEM_OPERATION_LOCK_REQUIRED,
+                           MM_BASE_MODEM_OPERATION_PRIORITY_OVERRIDE,
+                           (GAsyncReadyCallback)shutdown_disable_ready,
+                           ctx);
 }
 
 static gboolean
-foreach_remove (gpointer key,
-                MMDevice *device,
+foreach_remove (gpointer       key,
+                MMDevice      *device,
                 MMBaseManager *self)
 {
     MMBaseModem *modem;
@@ -689,7 +984,9 @@ foreach_remove (gpointer key,
 
 void
 mm_base_manager_shutdown (MMBaseManager *self,
-                          gboolean disable)
+                          gboolean       disable,
+                          gboolean       low_power,
+                          gboolean       remove)
 {
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_BASE_MANAGER (self));
@@ -698,7 +995,12 @@ mm_base_manager_shutdown (MMBaseManager *self,
     g_cancellable_cancel (self->priv->authp_cancellable);
 
     if (disable) {
-        g_hash_table_foreach (self->priv->devices, (GHFunc)foreach_disable, self);
+        DisableContext foreach_ctx = {
+            .self = self,
+            .low_power = low_power,
+            .remove = remove,
+        };
+        g_hash_table_foreach (self->priv->devices, (GHFunc)foreach_disable, &foreach_ctx);
 
         /* Disabling may take a few iterations of the mainloop, so the caller
          * has to iterate the mainloop until all devices have been disabled and
@@ -707,8 +1009,10 @@ mm_base_manager_shutdown (MMBaseManager *self,
         return;
     }
 
-    /* Otherwise, just remove directly */
-    g_hash_table_foreach_remove (self->priv->devices, (GHRFunc)foreach_remove, self);
+    if (remove) {
+        /* Otherwise, just remove directly */
+        g_hash_table_foreach_remove (self->priv->devices, (GHRFunc)foreach_remove, self);
+    }
 }
 
 guint32
@@ -747,7 +1051,7 @@ base_modem_sync_ready (MMBaseModem  *self,
         mm_obj_warn (self, "synchronization failed: %s", error->message);
         return;
     }
-    mm_obj_info (self, "synchronization finished");
+    mm_obj_msg (self, "synchronization finished");
 }
 
 void
@@ -767,8 +1071,12 @@ mm_base_manager_sync (MMBaseManager *self)
         modem = mm_device_peek_modem (MM_DEVICE (value));
 
         /* We just want to start the synchronization, we don't need the result */
-        if (modem)
-            mm_base_modem_sync (modem, (GAsyncReadyCallback)base_modem_sync_ready, NULL);
+        if (modem) {
+            mm_base_modem_sync (modem,
+                                MM_BASE_MODEM_OPERATION_LOCK_REQUIRED,
+                                (GAsyncReadyCallback)base_modem_sync_ready,
+                                NULL);
+        }
     }
 }
 
@@ -800,11 +1108,11 @@ set_logging_auth_ready (MMAuthProvider    *authp,
     GError *error = NULL;
 
     if (!mm_auth_provider_authorize_finish (authp, res, &error))
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
     else if (!mm_log_set_level (ctx->level, &error))
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
     else {
-        mm_obj_info (ctx->self, "logging: level '%s'", ctx->level);
+        mm_obj_msg (ctx->self, "logging: level '%s'", ctx->level);
         mm_gdbus_org_freedesktop_modem_manager1_complete_set_logging (
             MM_GDBUS_ORG_FREEDESKTOP_MODEM_MANAGER1 (ctx->self),
             ctx->invocation);
@@ -858,20 +1166,19 @@ scan_devices_auth_ready (MMAuthProvider *authp,
     GError *error = NULL;
 
     if (!mm_auth_provider_authorize_finish (authp, res, &error))
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
     else {
 #if defined WITH_UDEV
         if (!mm_context_get_test_no_udev ()) {
-            /* Otherwise relaunch device scan */
+            mm_obj_info (ctx->self, "processing user request to launch device scan");
             mm_base_manager_start (MM_BASE_MANAGER (ctx->self), TRUE);
             mm_gdbus_org_freedesktop_modem_manager1_complete_scan_devices (
                 MM_GDBUS_ORG_FREEDESKTOP_MODEM_MANAGER1 (ctx->self),
                 ctx->invocation);
         } else
 #endif
-            g_dbus_method_invocation_return_error_literal (
-                ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_UNSUPPORTED,
-                "Cannot request manual scan of devices: unsupported");
+            mm_dbus_method_invocation_return_error_literal (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_UNSUPPORTED,
+                                                            "Cannot request manual scan of devices: unsupported");
     }
 
     scan_devices_context_free (ctx);
@@ -942,7 +1249,7 @@ report_kernel_event_auth_ready (MMAuthProvider           *authp,
 out:
     if (error) {
         mm_obj_warn (ctx->self, "couldn't handle kernel event: %s", error->message);
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
     } else
         mm_gdbus_org_freedesktop_modem_manager1_complete_report_kernel_event (
             MM_GDBUS_ORG_FREEDESKTOP_MODEM_MANAGER1 (ctx->self),
@@ -1142,9 +1449,9 @@ remove_device_inhibition (MMBaseManager *self,
                 InhibitedDevicePortInfo *port_info;
 
                 port_info = (InhibitedDevicePortInfo *)(l->data);
-                mm_obj_info (self, "fake releasing port %s/%s during uninhibition...",
-                             mm_kernel_device_get_subsystem (port_info->kernel_port),
-                             mm_kernel_device_get_name (port_info->kernel_port));
+                mm_obj_msg (self, "fake releasing port %s/%s during uninhibition...",
+                            mm_kernel_device_get_subsystem (port_info->kernel_port),
+                            mm_kernel_device_get_name (port_info->kernel_port));
                 device_removed (self,
                                 mm_kernel_device_get_subsystem (port_info->kernel_port),
                                 mm_kernel_device_get_name (port_info->kernel_port));
@@ -1189,7 +1496,7 @@ inhibit_sender_lost (GDBusConnection          *connection,
                      const gchar              *sender_name,
                      InhibitSenderLostContext *lost_ctx)
 {
-    mm_obj_info (lost_ctx->self, "device inhibition teardown for uid '%s' (owner disappeared from bus)", lost_ctx->uid);
+    mm_obj_msg (lost_ctx->self, "device inhibition teardown for uid '%s' (owner disappeared from bus)", lost_ctx->uid);
     remove_device_inhibition (lost_ctx->self, lost_ctx->uid);
 }
 
@@ -1219,7 +1526,7 @@ device_inhibit_ready (MMDevice             *device,
     GError                   *error = NULL;
 
     if (!mm_device_inhibit_finish (device, res, &error)) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         inhibit_device_context_free (ctx);
         return;
     }
@@ -1244,7 +1551,7 @@ device_inhibit_ready (MMDevice             *device,
 
     g_hash_table_insert (ctx->self->priv->inhibited_devices, g_strdup (ctx->uid), info);
 
-    mm_obj_info (ctx->self, "device inhibition setup for uid '%s'", ctx->uid);
+    mm_obj_msg (ctx->self, "device inhibition setup for uid '%s'", ctx->uid);
 
     mm_gdbus_org_freedesktop_modem_manager1_complete_inhibit_device (
         MM_GDBUS_ORG_FREEDESKTOP_MODEM_MANAGER1 (ctx->self),
@@ -1259,19 +1566,20 @@ base_manager_inhibit_device (InhibitDeviceContext *ctx)
 
     device = find_device_by_physdev_uid (ctx->self, ctx->uid);
     if (!device) {
-        g_dbus_method_invocation_return_error (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND,
-                                               "No device found with uid '%s'", ctx->uid);
+        mm_dbus_method_invocation_return_error (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND,
+                                                "No device found with uid '%s'", ctx->uid);
         inhibit_device_context_free (ctx);
         return;
     }
 
     if (mm_device_get_inhibited (device)) {
-        g_dbus_method_invocation_return_error (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_IN_PROGRESS,
-                                               "Device '%s' is already inhibited", ctx->uid);
+        mm_dbus_method_invocation_return_error (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_IN_PROGRESS,
+                                                "Device '%s' is already inhibited", ctx->uid);
         inhibit_device_context_free (ctx);
         return;
     }
 
+    mm_obj_info (ctx->self, "processing user request to inhibit uid '%s'", ctx->uid);
     mm_device_inhibit (device,
                        (GAsyncReadyCallback) device_inhibit_ready,
                        ctx);
@@ -1287,14 +1595,15 @@ base_manager_uninhibit_device (InhibitDeviceContext *ctx)
     sender = g_dbus_method_invocation_get_sender (ctx->invocation);
     info = find_inhibited_device_info_by_physdev_uid (ctx->self, ctx->uid);
     if (!info || (g_strcmp0 (info->sender, sender) != 0)) {
-        g_dbus_method_invocation_return_error (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND,
-                                               "No inhibition found for uid '%s'", ctx->uid);
+        mm_dbus_method_invocation_return_error (ctx->invocation, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND,
+                                                "No inhibition found for uid '%s'", ctx->uid);
         inhibit_device_context_free (ctx);
         return;
     }
 
-    mm_obj_info (ctx->self, "device inhibition teardown for uid '%s'", ctx->uid);
+    mm_obj_info (ctx->self, "processing user request to uninhibit uid '%s'", ctx->uid);
     remove_device_inhibition (ctx->self, ctx->uid);
+    mm_obj_msg (ctx->self, "device inhibition teardown for uid '%s'", ctx->uid);
 
     mm_gdbus_org_freedesktop_modem_manager1_complete_inhibit_device (
         MM_GDBUS_ORG_FREEDESKTOP_MODEM_MANAGER1 (ctx->self),
@@ -1310,7 +1619,7 @@ inhibit_device_auth_ready (MMAuthProvider       *authp,
     GError *error = NULL;
 
     if (!mm_auth_provider_authorize_finish (authp, res, &error)) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        mm_dbus_method_invocation_take_error (ctx->invocation, error);
         inhibit_device_context_free (ctx);
         return;
     }
@@ -1360,13 +1669,14 @@ handle_set_profile (MmGdbusTest *skeleton,
     MMPlugin *plugin;
     MMDevice *device;
     gchar *physdev_uid;
+    gchar *physdev = NULL;
     GError *error = NULL;
 
-    mm_obj_info (self, "test profile set to: '%s'", id);
+    mm_obj_msg (self, "test profile set to: '%s'", id);
 
     /* Create device and keep it listed in the Manager */
     physdev_uid = g_strdup_printf ("/virtual/%s", id);
-    device = mm_device_new (physdev_uid, TRUE, TRUE, self->priv->object_manager);
+    device = mm_device_new (physdev_uid, physdev, TRUE, TRUE, self->priv->object_manager);
     g_hash_table_insert (self->priv->devices, physdev_uid, device);
 
     /* Grab virtual ports */
@@ -1394,15 +1704,18 @@ handle_set_profile (MmGdbusTest *skeleton,
         goto out;
     }
 
-    mm_obj_info (self, "modem for virtual device '%s' successfully created",
-                 mm_device_get_uid (device));
+    mm_obj_msg (self, "modem for virtual device '%s' successfully created",
+                mm_device_get_uid (device));
+
+    /* launch async modem initialization */
+    mm_device_initialize_modem (device);
 
 out:
 
     if (error) {
         mm_device_remove_modem (device);
         g_hash_table_remove (self->priv->devices, mm_device_get_uid (device));
-        g_dbus_method_invocation_return_gerror (invocation, error);
+        mm_dbus_method_invocation_return_gerror (invocation, error);
         g_error_free (error);
     } else
         mm_gdbus_test_complete_set_profile (skeleton, invocation);
@@ -1424,7 +1737,9 @@ log_object_build_id (MMLogObject *_self)
 
 MMBaseManager *
 mm_base_manager_new (GDBusConnection  *connection,
+#if !defined WITH_BUILTIN_PLUGINS
                      const gchar      *plugin_dir,
+#endif
                      gboolean          auto_scan,
                      MMFilterRule      filter_policy,
                      const gchar      *initial_kernel_events,
@@ -1439,7 +1754,9 @@ mm_base_manager_new (GDBusConnection  *connection,
                            NULL, /* cancellable */
                            error,
                            MM_BASE_MANAGER_CONNECTION,            connection,
+#if !defined WITH_BUILTIN_PLUGINS
                            MM_BASE_MANAGER_PLUGIN_DIR,            plugin_dir,
+#endif
                            MM_BASE_MANAGER_AUTO_SCAN,             auto_scan,
                            MM_BASE_MANAGER_FILTER_POLICY,         filter_policy,
                            MM_BASE_MANAGER_INITIAL_KERNEL_EVENTS, initial_kernel_events,
@@ -1489,10 +1806,12 @@ set_property (GObject      *object,
     case PROP_FILTER_POLICY:
         self->priv->filter_policy = g_value_get_flags (value);
         break;
+#if !defined WITH_BUILTIN_PLUGINS
     case PROP_PLUGIN_DIR:
         g_free (self->priv->plugin_dir);
         self->priv->plugin_dir = g_value_dup_string (value);
         break;
+#endif
     case PROP_INITIAL_KERNEL_EVENTS:
         g_free (self->priv->initial_kernel_events);
         self->priv->initial_kernel_events = g_value_dup_string (value);
@@ -1526,9 +1845,11 @@ get_property (GObject    *object,
     case PROP_FILTER_POLICY:
         g_value_set_flags (value, self->priv->filter_policy);
         break;
+#if !defined WITH_BUILTIN_PLUGINS
     case PROP_PLUGIN_DIR:
         g_value_set_string (value, self->priv->plugin_dir);
         break;
+#endif
     case PROP_INITIAL_KERNEL_EVENTS:
         g_value_set_string (value, self->priv->initial_kernel_events);
         break;
@@ -1592,7 +1913,11 @@ initable_init (GInitable     *initable,
         return FALSE;
 
     /* Create plugin manager */
-    self->priv->plugin_manager = mm_plugin_manager_new (self->priv->plugin_dir, self->priv->filter, error);
+    self->priv->plugin_manager = mm_plugin_manager_new (self->priv->filter,
+#if !defined WITH_BUILTIN_PLUGINS
+                                                        self->priv->plugin_dir,
+#endif
+                                                        error);
     if (!self->priv->plugin_manager)
         return FALSE;
 
@@ -1657,7 +1982,9 @@ finalize (GObject *object)
     MMBaseManager *self = MM_BASE_MANAGER (object);
 
     g_free (self->priv->initial_kernel_events);
+#if !defined WITH_BUILTIN_PLUGINS
     g_free (self->priv->plugin_dir);
+#endif
 
     g_hash_table_destroy (self->priv->inhibited_devices);
     g_hash_table_destroy (self->priv->devices);
@@ -1748,6 +2075,7 @@ mm_base_manager_class_init (MMBaseManagerClass *manager_class)
                             MM_FILTER_RULE_NONE,
                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+#if !defined WITH_BUILTIN_PLUGINS
     g_object_class_install_property
         (object_class, PROP_PLUGIN_DIR,
          g_param_spec_string (MM_BASE_MANAGER_PLUGIN_DIR,
@@ -1755,6 +2083,7 @@ mm_base_manager_class_init (MMBaseManagerClass *manager_class)
                               "Where to look for plugins",
                               NULL,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+#endif
 
     g_object_class_install_property
         (object_class, PROP_INITIAL_KERNEL_EVENTS,
