@@ -21,10 +21,14 @@
 #include <unistd.h>
 #include <string.h>
 
+#include "mm-iface-port-at.h"
 #include "mm-port-serial-at.h"
 #include "mm-log-object.h"
 
-G_DEFINE_TYPE (MMPortSerialAt, mm_port_serial_at, MM_TYPE_PORT_SERIAL)
+static void iface_port_at_init (MMIfacePortAtInterface *iface);
+
+G_DEFINE_TYPE_EXTENDED (MMPortSerialAt, mm_port_serial_at, MM_TYPE_PORT_SERIAL, 0,
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_PORT_AT, iface_port_at_init))
 
 enum {
     PROP_0,
@@ -54,32 +58,6 @@ struct _MMPortSerialAtPrivate {
 
 /*****************************************************************************/
 
-gchar *
-mm_port_serial_at_quote_string (const char *string)
-{
-    int len, i;
-    gchar *quoted, *pos;
-
-    if (string == NULL)
-        len = 0;
-    else
-        len = strlen (string);
-    quoted = g_malloc (3 + 3 * len); /* worst case */
-
-    pos = quoted;
-    *pos++ = '"';
-    for (i = 0 ; i < len; i++) {
-        if (string[i] < 0x20 || string[i] == '"' || string[i] == '\\')
-            pos += sprintf (pos, "\\%02X", string[i]);
-        else
-            *pos++ = string[i];
-    }
-    *pos++ = '"';
-    *pos++ = '\0';
-
-    return quoted;
-}
-
 void
 mm_port_serial_at_set_response_parser (MMPortSerialAt *self,
                                        MMPortSerialAtResponseParserFn fn,
@@ -340,40 +318,28 @@ at_command_to_byte_array (const char *command, gboolean is_raw, gboolean send_lf
     return buf;
 }
 
-const gchar *
+gchar *
 mm_port_serial_at_command_finish (MMPortSerialAt *self,
                                   GAsyncResult *res,
                                   GError **error)
 {
-    GString *str;
-
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return NULL;
-
-    str = (GString *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    return str->str;
-}
-
-static void
-string_free (GString *str)
-{
-    g_string_free (str, TRUE);
+    return g_task_propagate_pointer (G_TASK (res), error);
 }
 
 static void
 serial_command_ready (MMPortSerial *port,
                       GAsyncResult *res,
-                      GSimpleAsyncResult *simple)
+                      GTask *task)
 {
     GByteArray *response_buffer;
     GError *error = NULL;
     GString *response;
+    gchar *str;
 
     response_buffer = mm_port_serial_command_finish (port, res, &error);
     if (!response_buffer) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -384,11 +350,9 @@ serial_command_ready (MMPortSerial *port,
         g_byte_array_remove_range (response_buffer, 0, response_buffer->len);
     g_byte_array_unref (response_buffer);
 
-    g_simple_async_result_set_op_res_gpointer (simple,
-                                               response,
-                                               (GDestroyNotify)string_free);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    str = g_string_free (response, FALSE);
+    g_task_return_pointer (task, str, g_free);
+    g_object_unref (task);
 }
 
 void
@@ -401,8 +365,8 @@ mm_port_serial_at_command (MMPortSerialAt *self,
                            GAsyncReadyCallback callback,
                            gpointer user_data)
 {
-    GSimpleAsyncResult *simple;
     GByteArray *buf;
+    GTask *task;
 
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_PORT_SERIAL_AT (self));
@@ -415,10 +379,7 @@ mm_port_serial_at_command (MMPortSerialAt *self,
                                      TRUE));
     g_return_if_fail (buf != NULL);
 
-    simple = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        mm_port_serial_at_command);
+    task = g_task_new (self, NULL, callback, user_data);
 
     mm_port_serial_command (MM_PORT_SERIAL (self),
                             buf,
@@ -427,10 +388,43 @@ mm_port_serial_at_command (MMPortSerialAt *self,
                             is_raw, /* raw commands always run next, never queued last */
                             cancellable,
                             (GAsyncReadyCallback)serial_command_ready,
-                            simple);
+                            task);
     g_byte_array_unref (buf);
 }
 
+/*****************************************************************************/
+/* Integration with the Port AT interface */
+
+static gchar *
+iface_port_at_command_finish (MMIfacePortAt  *self,
+                              GAsyncResult   *res,
+                              GError        **error)
+{
+    return mm_port_serial_at_command_finish (MM_PORT_SERIAL_AT (self), res, error);
+}
+
+static void
+iface_port_at_command (MMIfacePortAt        *self,
+                       const gchar          *command,
+                       guint32               timeout_seconds,
+                       gboolean              is_raw,
+                       gboolean              allow_cached,
+                       GCancellable         *cancellable,
+                       GAsyncReadyCallback   callback,
+                       gpointer              user_data)
+{
+    mm_port_serial_at_command (MM_PORT_SERIAL_AT (self),
+                               command,
+                               timeout_seconds,
+                               is_raw,
+                               allow_cached,
+                               cancellable,
+                               callback,
+                               user_data);
+}
+
+/*****************************************************************************/
+
 static void
 debug_log (MMPortSerial *self,
            const gchar  *prefix,
@@ -630,6 +624,14 @@ finalize (GObject *object)
     G_OBJECT_CLASS (mm_port_serial_at_parent_class)->finalize (object);
 }
 
+static void
+iface_port_at_init (MMIfacePortAtInterface *iface)
+{
+    iface->check_support = NULL;
+    iface->command = iface_port_at_command;
+    iface->command_finish = iface_port_at_command_finish;
+}
+
 static void
 mm_port_serial_at_class_init (MMPortSerialAtClass *klass)
 {
